#!/bin/bash

# -------------------------------------------------------------------------------
# Filename:    check_weblogic.sh
# Revision:    1.0
# Date:        2006/10/24
# Author:      Luigi Balzano
# Email:       info@sistemistica.it
# Website:     www.sistemistica.it
# Description: Plugin for BEA Weblogic 8 heap memory check.
# Notes:       This plugin uses the "weblogic.Admin" facilities
# -------------------------------------------------------------------------------
# Copyright:   2006 (c) Luigi Balzano
# License:     GPL
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# you should have received a copy of the GNU General Public License
# along with this program (or with Nagios); if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA
# -------------------------------------------------------------------------------
# Credits go to Ethan Galstad for coding Nagios
# If any changes are made to this script, please mail me a copy of the changes
#

PROGNAME=`basename $0`
PROGPATH=`echo $0 | /bin/sed -e 's,[\\/][^\\/][^\\/]*$,,'`
REVISION=`echo '$Revision: 1.0 $' | sed -e 's/[^0-9.]//g'`
TIMESTAMP=`date +%Y%m%d%H%M%S | sed 's/\(....\)\(..\)\(..\)\(..\)\(..\)\(..\)/\1-\2-\3 \4:\5:\6/'`

. $PROGPATH/utils.sh

print_usage() {
        echo "Usage: $PROGNAME -h"
        echo "Usage: $PROGNAME -v"
        echo "Usage: $PROGNAME -f CONFIGFILE -c CRITICAL LEVELS -w WARNING LEVELS"
        echo ""
        echo "Path of CONFIGFILE must be relative to the program path"
        echo "(e.g. no path needed if config file is in the program dir)"
        echo ""
        echo "CRITICAL LEVELS and WARNING LEVELS must be specified in this form:"
        echo "HEAPFREECURR,LASTGCTIME,TOTALGCCOUNT"
        echo ""
        echo "HEAPFREECURR = Min number of free heap (in bytes) under what alarm is triggered (-1 exclude parameter)"
        echo "LASTGCTIME = Max amount of time (in milliseconds) spent in last Garbage Collection above what alarm is triggered (-1 exclude parameter)"
        echo "TOTALGCCOUNT = Total number of Garbage Collection made above what alarm is triggered (-1 exclude parameter)"
}

print_help() {
        print_revision $PROGNAME $REVISION
        echo ""
        print_usage
        echo ""
        echo "Plugin for BEA Weblogic 8 heap memory check."
        echo ""
        support
}

# No command line arguments are supplied for this script. Print help

if [ "$#" -eq "0" ]; then
        print_usage
        exit $STATE_UNKNOWN
fi

# Print help with --help

if [[ ("$#" -eq "1") && ("$1" = "--help") ]]; then
        print_help
        exit $STATE_UNKNOWN
fi

unset CONFIGFILE
unset CRITICALS
unset WARNINGS

while getopts "vhf:c:w:" Option
do
  case $Option in
    h     ) print_help
            exit $STATE_UNKNOWN;;
    v     ) print_revision $PROGNAME $REVISION
            exit $STATE_UNKNOWN;;
    f     ) CONFIGFILE=$OPTARG;;
    c     ) CRITICALS=$OPTARG;;
    w     ) WARNINGS=$OPTARG;;
    *     ) print_usage
            exit $STATE_UNKNOWN;;   # DEFAULT
  esac
done

if [[ (! -n "$CONFIGFILE") || (! -n "$CRITICALS") || (! -n "$WARNINGS") ]]
  then
        echo "Unspecified argument"
        print_usage
        exit $STATE_UNKNOWN
fi

if [ -f $PROGPATH/$CONFIGFILE ]
  then
        . "$PROGPATH/$CONFIGFILE"
        export WL_HOME
        export JAVA_HOME
        export CLASSPATH
  else
        echo "Config file not found"
        print_usage
        exit $STATE_UNKNOWN
fi

OPTION="$CRITICALS"
COUNT=0
until [ -z "$OPTION" ]
do
  COUNT=`expr $COUNT + 1`
  OPTLEN=${#OPTION}
  NEXTCOMMA=`expr index "$OPTION" ,`
  if [ "$NEXTCOMMA" -eq "0" ]
    then
      NEXTCOMMA=$OPTLEN
  fi
  TOKEN=${OPTION:0:NEXTCOMMA}
  TOKEN=${TOKEN%,}
  case $COUNT in
	1) CRIT_HEAPFREECURR=$TOKEN;;
	2) CRIT_LASTGCTIME=$TOKEN;;
	3) CRIT_TOTALGCCOUNT=$TOKEN;;
	*) echo "Syntax error for CRITICALS argument"
       print_usage
       exit $STATE_UNKNOWN;;
  esac
  OPTION=${OPTION:NEXTCOMMA:`expr $OPTLEN - $NEXTCOMMA`}
done
if [ "$COUNT" -lt "3" ]
  then
    echo "Syntax error for CRITICALS argument"
    print_usage
    exit $STATE_UNKNOWN
fi

OPTION="$WARNINGS"
COUNT=0
until [ -z "$OPTION" ]
do
  COUNT=`expr $COUNT + 1`
  OPTLEN=${#OPTION}
  NEXTCOMMA=`expr index "$OPTION" ,`
  if [ "$NEXTCOMMA" -eq "0" ]
    then
      NEXTCOMMA=$OPTLEN
  fi
  TOKEN=${OPTION:0:NEXTCOMMA}
  TOKEN=${TOKEN%,}
  case $COUNT in
	1) WARN_HEAPFREECURR=$TOKEN;;
	2) WARN_LASTGCTIME=$TOKEN;;
	3) WARN_TOTALGCCOUNT=$TOKEN;;
	*) echo "Syntax error for WARNINGS argument"
       print_usage
       exit $STATE_UNKNOWN;;
  esac
  OPTION=${OPTION:NEXTCOMMA:`expr $OPTLEN - $NEXTCOMMA`}
done
if [ "$COUNT" -lt "3" ]
  then
    echo "Syntax error for WARNINGS argument"
    print_usage
    exit $STATE_UNKNOWN
fi

nc -z $INSTANCE $PORT
LISTEN=$?

if [ "$LISTEN" -ne "0" ]
  then
    echo "CRITICAL - Unable to connect to instance $INSTANCE port $PORT"
    exit $STATE_CRITICAL
fi

HEAPFREECURR="-1"
LASTGCTIME="-1"
TOTALGCCOUNT="-1"

RESULT=`$JAVA_HOME/bin/java weblogic.Admin -url t3://$INSTANCE:$PORT -username $USERNAME -password $PASSWORD GET -pretty -type JRockitRuntime -property HeapFreeCurrent -property LastGCEnd -property LastGCStart -property TotalGarbageCollectionCount | awk '
BEGIN { HeapFreeCurrent = -1; LastGCEnd = -1; LastGCStart = -1; TotalGarbageCollectionCount = -1; LastGCTime = -1; }
/HeapFreeCurrent/ { HeapFreeCurrent = $2; }
/LastGCEnd/ { LastGCEnd = $2; }
/LastGCStart/ { LastGCStart = $2; }
/TotalGarbageCollectionCount/ { TotalGarbageCollectionCount = $2; }
END {
  if ((LastGCEnd > -1) && (LastGCStart > -1)) {
    LastGCTime = LastGCEnd - LastGCStart;
  }
  printf ("%s|%s|%s\n", HeapFreeCurrent, LastGCTime, TotalGarbageCollectionCount);
}
'`
#echo "$RESULT"

HEAPFREECURR=$(echo $RESULT | cut -d"|" -f1)
LASTGCTIME=$(echo $RESULT | cut -d"|" -f2)
TOTALGCCOUNT=$(echo $RESULT | cut -d"|" -f3)
#echo "$HEAPFREECURR|$LASTGCTIME|$TOTALGCCOUNT"

#### Check criticals ####
if [ "$(echo "($CRIT_HEAPFREECURR != -1) && ($HEAPFREECURR <= $CRIT_HEAPFREECURR)" | bc)" -eq "1" ]
  then
    echo "CRITICAL - $HEAPFREECURR bytes of current free heap memory (TimeStamp=$TIMESTAMP, HeapFreeCurrent=$HEAPFREECURR, LastGCTime=$LASTGCTIME, TotalGCCount=$TOTALGCCOUNT)"
    exit $STATE_CRITICAL
fi

if [ "$(echo "($CRIT_LASTGCTIME != -1) && ($LASTGCTIME >= $CRIT_LASTGCTIME)" | bc)" -eq "1" ]
  then
    echo "CRITICAL - $LASTGCTIME milliseconds needed for last Garbage Collection (TimeStamp=$TIMESTAMP, HeapFreeCurrent=$HEAPFREECURR, LastGCTime=$LASTGCTIME, TotalGCCount=$TOTALGCCOUNT)"
    exit $STATE_CRITICAL
fi

if [ "$(echo "($CRIT_TOTALGCCOUNT != -1) && ($TOTALGCCOUNT >= $CRIT_TOTALGCCOUNT)" | bc)" -eq "1" ]
  then
    echo "CRITICAL - $TOTALGCCOUNT Garbage Collection have been made up to now (TimeStamp=$TIMESTAMP, HeapFreeCurrent=$HEAPFREECURR, LastGCTime=$LASTGCTIME, TotalGCCount=$TOTALGCCOUNT)"
    exit $STATE_CRITICAL
fi

#### Check warnings ####
if [ "$(echo "($WARN_HEAPFREECURR != -1) && ($HEAPFREECURR <= $WARN_HEAPFREECURR)" | bc)" -eq "1" ]
  then
    echo "WARNING - $HEAPFREECURR bytes of current free heap memory (TimeStamp=$TIMESTAMP, HeapFreeCurrent=$HEAPFREECURR, LastGCTime=$LASTGCTIME, TotalGCCount=$TOTALGCCOUNT)"
    exit $STATE_WARNING
fi

if [ "$(echo "($WARN_LASTGCTIME != -1) && ($LASTGCTIME >= $WARN_LASTGCTIME)" | bc)" -eq "1" ]
  then
    echo "WARNING - $LASTGCTIME milliseconds needed for last Garbage Collection (TimeStamp=$TIMESTAMP, HeapFreeCurrent=$HEAPFREECURR, LastGCTime=$LASTGCTIME, TotalGCCount=$TOTALGCCOUNT)"
    exit $STATE_WARNING
fi

if [ "$(echo "($WARN_TOTALGCCOUNT != -1) && ($TOTALGCCOUNT >= $WARN_TOTALGCCOUNT)" | bc)" -eq "1" ]
  then
    echo "WARNING - $TOTALGCCOUNT Garbage Collection have been made up to now (TimeStamp=$TIMESTAMP, HeapFreeCurrent=$HEAPFREECURR, LastGCTime=$LASTGCTIME, TotalGCCount=$TOTALGCCOUNT)"
    exit $STATE_WARNING
fi

echo "OK - TimeStamp=$TIMESTAMP, HeapFreeCurrent=$HEAPFREECURR, LastGCTime=$LASTGCTIME, TotalGCCount=$TOTALGCCOUNT"
exit $STATE_OK
